###############################################################################
package SOCC::File::State;

use strict;
use English;
our $VERSION = 0.04;



sub new {
	my ($class) = shift @_;
	my (%args) = (dumper => 1, @_);
	
	my $dumper = $args{dumper};
	delete $args{dumper};
	
	my $self;
	if ($dumper) {
		$self = SOCC::File::State::Dumper->new(%args);
	} else {
		$self = SOCC::File::State::Storable->new(%args);
	}
	
	return $self;
}



###############################################################################
package SOCC::File::State::Storable;

use strict;
use English;
use Storable ('lock_store', 'lock_retrieve');



{
	my $idx = 0;
	use constant PATH => $idx++;
	use constant VARIABLES => $idx++;
}



sub new {
	my ($class, %args) = @_;
	
	my $self = bless([], $class);
	$self->[PATH] = $args{path};
	delete $args{path};
	$self->[VARIABLES] = {};
	
	$self->read();
	foreach my $method (keys %args) {
		$self->$method($args{$method});
	}
	
	return $self;
}



sub variable {
	my ($self, $name, $newval) = @_;
	
	if (defined $newval) {
		$self->[VARIABLES]->{$name} = $newval;
	}
	return $self->[VARIABLES]->{$name};
}



sub read {
	my ($self) = @_;
	my $ok = '';
	
	foreach my $filename ($self->[PATH], $self->[PATH] . '.1',
		$self->[PATH] . '.2') {
		if (-r $filename) {
			eval { $self->[VARIABLES] = lock_retrieve($filename); };
			if ( ($ok = $EVAL_ERROR eq '') ) {
				last;
			}
		}
	}
	
	return $ok;
}



sub write {
	my ($self, %args) = @_;
	my $ok = '';
	
	eval {
		lock_store($self->[VARIABLES], $self->[PATH] . '.tmp');
	};
	if ( ($ok = $EVAL_ERROR eq '') ) {
		unlink($self->[PATH] . '.2');
		rename($self->[PATH] . '.1', $self->[PATH] . '.2');
		rename($self->[PATH], $self->[PATH] . '.1');
		$ok = rename($self->[PATH] . '.tmp', $self->[PATH]);
	}
	
	return $ok;
}



our $AUTOLOAD;
sub AUTOLOAD {
	my ($self, $newval) = @_;
	
	my $method = $AUTOLOAD;
	$method =~ s/.*:://;
	
	if ($method ne 'DESTROY') {
		return $self->variable($method, $newval);
	}
}



###############################################################################
package SOCC::File::State::Dumper;

use strict;
use English;
use Data::Dumper;
use Fcntl (':flock');


{
	my $idx = 0;
	use constant PATH => $idx++;
	use constant VARIABLES => $idx++;
}


sub new {
	my ($class, %args) = @_;
	
	my $self = bless([], $class);
	$self->[PATH] = $args{path};
	delete $args{path};
	$self->[VARIABLES] = {};
	
	$self->read();
	foreach my $method (keys %args) {
		$self->$method($args{$method});
	}
	
	return $self;
}


sub variable {
	my ($self, $name, $newval) = @_;
	
	if (defined $newval) {
		$self->[VARIABLES]->{$name} = $newval;
	}
	return $self->[VARIABLES]->{$name};
}

sub read {
	my ($self) = @_;
	my $ok = '';
	
	foreach my $filename ($self->[PATH], $self->[PATH] . '.1', 
		$self->[PATH] . '.2') {
		open(FH, "<", $filename) or next;
		flock(FH, LOCK_SH) or next;
		
		select FH;
		local $INPUT_RECORD_SEPARATOR;
		my $contents = <FH>;
		close(FH);
		
		my $variables;
		eval $contents;
		$EVAL_ERROR eq '' or next;
		
		$self->[VARIABLES] = $variables;
		$ok = 1;
		last;
	}
	
	return $ok;
}


sub write {
	my ($self, %args) = @_;
	my $ok = '';
	
	
	if ($ok = open(FH, '> ' . $self->[PATH] . '.tmp') && flock(FH, LOCK_EX)) {
		my $dd = Data::Dumper->new([$self->[VARIABLES]], ['variables']);
		$dd->Indent(1);
		if (($ok = print(FH $dd->Dump()))) {
			if (($ok = close(FH))) {
				unlink($self->[PATH] . '.2');
				rename($self->[PATH] . '.1', $self->[PATH] . '.2');
				rename($self->[PATH], $self->[PATH] . '.1');
				$ok = rename($self->[PATH] . '.tmp', $self->[PATH]);
			}
		}
	}
	
	return $ok;
}


our $AUTOLOAD;
sub AUTOLOAD {
	my ($self, $newval) = @_;
	
	my $method = $AUTOLOAD;
	$method =~ s/.*:://;
	
	if ($method ne 'DESTROY') {
		return $self->variable($method, $newval);
	}
}


1;
