#!/usr/bin/perl

use strict;
use warnings;
use English;
use Test::More tests => 13;
use Class::Struct FileStatus => {mtime => '$', size => '$'};


BEGIN {
	mkdir('tmp');
	unlink('tmp/test_statefile');
	unlink('tmp/test_statefile.1');
	unlink('tmp/test_statefile.2');
	use_ok('SOCC::File::State');
}


# dumper defaults to true if not supplied.
for my $dumper ('', 1) {
	
	# Test Setup.
	unlink('tmp/test_statefile');
	unlink('tmp/test_statefile.1');
	unlink('tmp/test_statefile.2');
	
	{
		my $file_status = FileStatus->new(mtime => 1234, size => 999);
		my $daemon_status = 'INITALIZING';
		
		# Storing and retrieving data.  
		my $statefile = SOCC::File::State->new(path => 'tmp/test_statefile', 
			dumper => $dumper);
		$statefile->daemon_status('INITALIZING');
		$statefile->variable('file_status', $file_status);
		my $ok = $statefile->write();
		ok ($ok);
		
		undef $statefile;
		$statefile = SOCC::File::State->new(path => 'tmp/test_statefile', 
			dumper => $dumper);
		ok ($statefile->daemon_status() eq $daemon_status);
		ok ($statefile->variable('file_status')->isa('FileStatus') &&
			$statefile->variable('file_status')->mtime == $file_status->mtime &&
			$statefile->variable('file_status')->size == $file_status->size);
	}



	{
		# old copies of the state file are copied for error recovery, see below:
		# mv tmp/test_statefile.1 tmp/test_statefile.2
		# mv tmp/test_statefile tmp/test_statefile.1
		# new data written to tmp/test_statefile
		my $statefile = SOCC::File::State->new(path => 'tmp/test_statefile',
			daemon_status=>'INITALIZING');
		$statefile->write();
		
		$statefile->daemon_status('RUNNING');
		$statefile->write();
		
		$statefile->daemon_status('STOPPING');
		$statefile->write();
		
		# When creating a state object the newest data file is used in
		# preference to the oldest.  If a file contains invalid data,
		# it will be skipped.
		undef $statefile;
		$statefile = SOCC::File::State->new(path => 'tmp/test_statefile');
		ok($statefile->daemon_status() eq 'STOPPING');
		
		undef $statefile;
		unlink 'tmp/test_statefile';
		$statefile = SOCC::File::State->new(path => 'tmp/test_statefile');
		ok($statefile->daemon_status() eq 'RUNNING');
		
		undef $statefile;
		unlink 'tmp/test_statefile.1';
		$statefile = SOCC::File::State->new(path => 'tmp/test_statefile');
		ok($statefile->daemon_status() eq 'INITALIZING');
	}
}


